package ntnu.ui;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Align;
import android.view.View;

/**
 * GraphView creates a scaled line or bar graph with x and y axis labels. 
 * @author Arno den Hond
 *
 */
public class GraphView extends View {

	public static boolean BAR = true;
	public static boolean LINE = false;

	private int selected = -1;
	
	public int getSelected() {
		return selected;
	}



	public void setSelected(int selected) {
		this.selected = selected;
	}

	private Paint paint;
	private float[] values;
	private String[] horlabels;
	private String[] verlabels;
	public float[] getValues() {
		return values;
	}

	
	
	public void setValues(float[] values) {
		this.values = values;
	}

	public String[] getHorlabels() {
		return horlabels;
	}

	public void setHorlabels(String[] horlabels) {
		this.horlabels = horlabels;
	}

	public String[] getVerlabels() {
		return verlabels;
	}

	public void setVerlabels(String[] verlabels) {
		this.verlabels = verlabels;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public boolean isType() {
		return type;
	}

	public void setType(boolean type) {
		this.type = type;
	}

	private String title;
	private boolean type;

	public GraphView(Context context, float[] values, String title, String[] horlabels, String[] verlabels, boolean type) {
		super(context);
		if (values == null)
			values = new float[0];
		else
			this.values = values;
		if (title == null)
			title = "";
		else
			this.title = title;
		if (horlabels == null)
			this.horlabels = new String[0];
		else
			this.horlabels = horlabels;
		if (verlabels == null)
			this.verlabels = new String[0];
		else
			this.verlabels = verlabels;
		this.type = type;
		paint = new Paint();
	}

	@Override
	protected void onDraw(Canvas canvas) {
		
		
		
		float border = 20;
		float horstart = border * 2;
		float height = getHeight();
		float width = getWidth() - 1;
		float max = getMax();
		float min = getMin();
		float diff = max - min;
		float graphheight = height - (2 * border);
		float graphwidth = width - (2 * border);

		paint.setTextAlign(Align.LEFT);
		int vers = verlabels.length - 1;
		for (int i = 0; i < verlabels.length; i++) {
			paint.setColor(Color.DKGRAY);
			float y = ((graphheight / vers) * i) + border;
			canvas.drawLine(horstart, y, width, y, paint);
			paint.setColor(Color.BLACK);
			canvas.drawText(verlabels[i], 0, y, paint);
		}
		int hors = horlabels.length - 1;
		for (int i = 0; i < horlabels.length; i++) {
			paint.setColor(Color.DKGRAY);
			float x = ((graphwidth / hors) * i) + horstart;
			canvas.drawLine(x, height - border, x, border, paint);
			paint.setTextAlign(Align.CENTER);
			if (i==horlabels.length-1)
				paint.setTextAlign(Align.RIGHT);
			if (i==0)
				paint.setTextAlign(Align.LEFT);
			paint.setColor(Color.BLACK);
			if(hors==0)
			{
				canvas.drawText(horlabels[i], getWidth()/2, height - 4, paint);
			}
			else 
			{
				canvas.drawText(horlabels[i], x, height - 4, paint);
			}
		}

		paint.setTextAlign(Align.CENTER);
		canvas.drawText(title, (graphwidth / 2) + horstart, border - 4, paint);

		if (max != min) {
			paint.setColor(Color.BLUE);
			if (type == BAR) {
				float datalength = values.length;
				float colwidth = (width - (2 * border)) / datalength;
				
				if(values.length==1)
				{
					for (int i = 0; i < values.length; i++) {
						float val = values[i] - min;
						System.out.println("this is value:" +  values[i]);
						float rat = val / diff;
						float h = graphheight * rat;
						if(i!=selected)
						{ 
							paint.setColor(Color.BLUE);
							canvas.drawRect((i * colwidth) + horstart+60, (border - h) + graphheight, ((i * colwidth) + horstart) + (colwidth - 1)-60, height - (border - 1), paint);
						}
						else
						{
							paint.setColor(Color.DKGRAY);
							canvas.drawRect((i * colwidth) + horstart+60, (border - h) + graphheight, ((i * colwidth) + horstart) + (colwidth - 1)-60, height - (border - 1), paint);
						}
					}
				}
				else 
				{
				
				
				for (int i = 0; i < values.length; i++) {
					float val = values[i] - min;
					System.out.println("this is value:" +  values[i]);
					float rat = val / diff;
					float h = graphheight * rat;
					if(i!=selected)
					{ 
						paint.setColor(Color.BLUE);
						canvas.drawRect((i * colwidth) + horstart, (border - h) + graphheight, ((i * colwidth) + horstart) + (colwidth - 1), height - (border - 1), paint);
					}
					else
					{
						paint.setColor(Color.DKGRAY);
						canvas.drawRect((i * colwidth) + horstart, (border - h) + graphheight, ((i * colwidth) + horstart) + (colwidth - 1), height - (border - 1), paint);
					}
				}
				
			
			}}}
		
	}

	private float getMax() {
		float largest = Integer.MIN_VALUE;
		for (int i = 0; i < values.length; i++)
			if (values[i] > largest)
				largest = values[i];
		System.out.println(largest);
		return 1;
	}

	private float getMin() {
		float smallest = Integer.MAX_VALUE;
		for (int i = 0; i < values.length; i++)
			if (values[i] < smallest)
				smallest = values[i];
		//return smallest;
		return 0;
	}
	
	


}
