package dao.factory;

import dao.AttributeDAO;
import dao.CitynodeDAO;
import dao.PreferenceDAO;
import dao.RatingDAO;
import dao.RecommendationDAO;
import dao.UserDAO;
import dao.UserModelDAO;
import dao.VisitDAO;

public abstract class DAOFactory {
	
    /**
     * Creates a standalone DAOFactory that returns unmanaged DAO
     * beans for use in any environment Hibernate has been configured
     * for. Uses HibernateUtil/SessionFactory and Hibernate context
     * propagation (CurrentSessionContext), thread-bound or transaction-bound,
     * and transaction scoped.
     */
    public static final Class HIBERNATE = dao.factory.HibernateDAOFactory.class;

    /**
     * Factory method for instantiation of concrete factories.
     */
    public static DAOFactory instance(Class factory) {
        try {
            return (DAOFactory)factory.newInstance();
        } catch (Exception ex) {
            throw new RuntimeException("Couldn't create DAOFactory: " + factory);
        }
    }

    // Add your DAO interfaces here
    public abstract CitynodeDAO getCitynodeDAO();
   
    public abstract RecommendationDAO getRecommendationDAO();
    
    public abstract AttributeDAO getAttributeDAO();
	
    public abstract UserDAO getUserDAO();
    
    public abstract UserModelDAO getUserModelDAO();
    
    public abstract PreferenceDAO getPreferenceDAO();
    
    public abstract VisitDAO getVisitDAO();
    
    public abstract RatingDAO getRatingDAO(); 
    
    
}
