package dao.factory;

import org.hibernate.Session;
import org.hibernate.search.Search;

import util.HibernateUtil;

import dao.AttributeDAO;
import dao.CitynodeDAO;
import dao.PreferenceDAO;
import dao.RatingDAO;
import dao.RecommendationDAO;
import dao.UserDAO;
import dao.UserModelDAO;
import dao.VisitDAO;
import dao.impl.AttributeDAOImpl;
import dao.impl.CitynodeDAOimpl;
import dao.impl.GenericDAOimpl;
import dao.impl.PreferenceDAOimpl;
import dao.impl.RatingDAOimpl;
import dao.impl.RecommendationDAOimpl;
import dao.impl.UserDAOimpl;
import dao.impl.UserModelDAOimpl;
import dao.impl.VisitDAOimpl;

public class HibernateDAOFactory extends DAOFactory{
	
	private Session session; 
	
    public CitynodeDAO getCitynodeDAO() {
        return (CitynodeDAO)instantiateDAO(CitynodeDAOimpl.class);
    }

   

    private GenericDAOimpl instantiateDAO(Class daoClass) {
        try {
            GenericDAOimpl dao = (GenericDAOimpl)daoClass.newInstance();
            dao.setSession(getSession());
            dao.setFullTextSession(Search.getFullTextSession(getSession()));
            return dao;
        } catch (Exception ex) {
            throw new RuntimeException("Can not instantiate DAO: " + daoClass, ex);
        }
    }

    // You could override this if you don't want HibernateUtil for lookup
    protected Session getCurrentSession() {
        return HibernateUtil.getSessionFactory().getCurrentSession();
    }

    protected Session getSession()
    {
    	if(session==null)
    		session = HibernateUtil.getSessionFactory().openSession(); 
    	return session;  
    		
    }

    
    
	@Override
	public RecommendationDAO getRecommendationDAO() {
		// TODO Auto-generated method stub
		return  (RecommendationDAO) instantiateDAO(RecommendationDAOimpl.class);
	}



	@Override
	public AttributeDAO getAttributeDAO() {
		// TODO Auto-generated method stub
		return (AttributeDAO)instantiateDAO(AttributeDAOImpl.class); 
	}



	@Override
	public UserDAO getUserDAO() {
		// TODO Auto-generated method stub
		return (UserDAO)instantiateDAO(UserDAOimpl.class);
	}



	@Override
	public UserModelDAO getUserModelDAO() {
		// TODO Auto-generated method stub
		return (UserModelDAO)instantiateDAO(UserModelDAOimpl.class);
	}



	@Override
	public PreferenceDAO getPreferenceDAO() {
		return (PreferenceDAO)instantiateDAO(PreferenceDAOimpl.class);
	}

	@Override
	public VisitDAO getVisitDAO() {
		return (VisitDAO)instantiateDAO(VisitDAOimpl.class);
	}
	
	
	
	//TODO: Do something better
	public void closeSession()
	{
		this.session.close();
		this.session = null;
	}



	@Override
	public RatingDAO getRatingDAO() {
		// TODO Auto-generated method stub
		return (RatingDAO)instantiateDAO(RatingDAOimpl.class);
	}
	
}
