package dao.impl;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.sql.BatchUpdateException;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.LockMode;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Example;
import org.hibernate.search.FullTextSession;

import dao.GenericDAO;

public abstract class GenericDAOimpl<T, ID extends Serializable>
	implements GenericDAO<T, ID> {

	private Class<T> persistentClass;
	private Session session;
	private FullTextSession fullTextSession;
	


	public GenericDAOimpl() {
		this.persistentClass = (Class<T>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
}

	@SuppressWarnings("unchecked")
	public void setSession(Session s) {
		this.session = s;
	}

	protected Session getSession() {
		if (session == null)
			throw new IllegalStateException("Session has not been set on DAO before usage");
		return session;
}

	public Class<T> getPersistentClass() {
		return persistentClass;
	}

	@SuppressWarnings("unchecked")
	public T findById(ID id, boolean lock) {
	
		Transaction tx = getSession().beginTransaction(); 
		T entity;
	
	
 
		if (lock)
			entity = (T) getSession().load(getPersistentClass(), id, LockMode.UPGRADE);
		else
			entity = (T) getSession().load(getPersistentClass(), id);
		
		tx.commit();
		

	return entity;
}

	@SuppressWarnings("unchecked")
	public List<T> findAll() {
		return findByCriteria();
	}

	@SuppressWarnings("unchecked")
	public List<T> findByExample(T exampleInstance, String[] excludeProperty) 
	{
		
		Transaction tx = getSession().beginTransaction(); 
		
		Criteria crit = getSession().createCriteria(getPersistentClass());
		Example example =  Example.create(exampleInstance);
		for (String exclude : excludeProperty) {
			example.excludeProperty(exclude);
		}
		crit.add(example);
		List <T> list = crit.list();
		
		tx.commit();
		
		return list; 
	}

	@SuppressWarnings("unchecked")
	public T makePersistent(T entity)  {
		Transaction tx = getSession().beginTransaction(); 
		
			getSession().saveOrUpdate(entity);
			tx.commit();

		return entity;
	}

	public void makeTransient(T entity) {
		Transaction tx = getSession().beginTransaction(); 
		getSession().delete(entity);
		tx.commit(); 
	}

	public void flush() {
		getSession().flush();
	}

	public void clear() {
		getSession().clear();
	}

/**
* Use this inside subclasses as a convenience method.
*/
	@SuppressWarnings("unchecked")
	protected List<T> findByCriteria(Criterion... criterion) 
	{
		Transaction tx = getSession().beginTransaction(); 
		
		Criteria crit = getSession().createCriteria(getPersistentClass());
		for (Criterion c : criterion) {
			crit.add(c);
		}
		List <T> list =  (crit.list());
		
		tx.commit();
		
		return list; 
	}
	
	
	protected FullTextSession getFullTextSession() {
		return fullTextSession;
	}

	public void setFullTextSession(FullTextSession fullTextSession) {
		this.fullTextSession = fullTextSession;
	}
	

}



