package matrixPacket;


import java.util.ArrayList;
import model.Citynode;
import Jama.Matrix;

public class RecommenderAlgorithm {
	
	Matrix Sij;
	Matrix Wab;
	ArrayList<User> users;
	ArrayList<Item> items;
	double noise=0.8;
	double meanWeight=0.;
	double varianceWeight=0.;
	double tradeoff=.001;
	private double likeliTradeOff=.709;
	
	
	// Prior-code
	
	public RecommenderAlgorithm(ArrayList<User> users, ArrayList<Item> items)
	{
		this.users=users;
		this.items=items;
	}
	
	public void createProbabilisticModel()
	{
		if(users.size()<1 || items.size()<1) return;
		int uRow = users.get(0).getUserFeatures().getRowDimension();
		int iRow = items.get(0).getItemFeatures().getRowDimension();
		int dim = uRow*iRow;
		
		Matrix A = new Matrix(dim, dim);
	    for (int i = 0; i < users.size(); i++) {
	    	Matrix uZ = new Matrix(iRow,1);
	    	for (int j = 0; j < users.get(i).getRatings().size(); j++) {
				uZ = uZ.plus(items.get(users.get(i).getRatings().get(j).item).getItemFeatures());
				if(i==4) uZ.print(iRow, 1);
			}
	    	uZ = uZ.times(1./(double)users.get(i).getRatings().size());
	    	//if(i==4) uZ.print(iRow, 1);
			for (int j = 0; j < users.get(i).getRatings().size(); j++) {
				Item item=items.get(users.get(i).getRatings().get(j).item);
				Matrix temp = item.getItemFeatures().times((item.getItemFeatures().minus(uZ)).transpose());
				A = A.plus(temp.tensorProduct(users.get(i).getUserFeatures().times(users.get(i).getUserFeatures().transpose())));
			}
			//A.print(30, 2);
		}
	    Matrix B = new Matrix(dim,1);
	    for (int i = 0; i < users.size(); i++) {
	    	Matrix uZ = new Matrix(iRow,1);
	    	for (int j = 0; j < users.get(i).getRatings().size(); j++) {
				uZ = uZ.plus(items.get(users.get(i).getRatings().get(j).item).getItemFeatures());
			}
	    	uZ = uZ.times(1./(double)users.get(i).getRatings().size());
			for (int j = 0; j < users.get(i).getRatings().size(); j++) {
				Item item=items.get(users.get(i).getRatings().get(j).item);
				Matrix temp = (item.getItemFeatures().minus(uZ)).times(users.get(i).getRatings().get(j).rating);
				B = B.plus(temp.tensorProduct(users.get(i).getUserFeatures()));
			}
		}
	    
	    Matrix w = (A.plus((Matrix.identity(dim, dim).times(tradeoff/2.)))).inverse().times(B);
	    
	    
	    // Calculate probable score
	    Wab=w;
	    
	}
	
	public void optimizeWeights(){
		if (Wab==null) {
			Wab=new Matrix(users.get(0).getUserFeatures().getRowDimension(),items.get(0).getItemFeatures().getRowDimension());
		}
		double precision = 0.1;
		updateMeanAndVariance();
		double newE=999999.;
		double oldE =1000.;
		double gradient;
		double temp = 0.;
		double lastTemp = 999.;
		Matrix oldW;
		double flip = 1.0;
		int iterations = 0;
		//int[] shuffle = new int[users.size()];
		do{
			
			oldE=newE;
			newE=likelihoodFunctionC();
			System.out.println("likelihood is now "+newE);
			oldW=Wab.copy();
			for(int b = 0; b<Wab.getRowDimension();b++) {
				for(int a = 0; a<Wab.getColumnDimension();a++) {
					gradient = Wab.get(b, a)*(1./likeliTradeOff);
					for (int i = 0; i < users.size(); i++) {
						User user = users.get(i);
						for (Rating rating : user.getRatings()) {
//							System.out.println("Rating: " + rating.item + " -- " + rating.rating);
							//double sij = calculateSijAlt(i, rating.item);
							//double derived = (sij*sij-rating.rating*rating.rating)/((sij*sij)*(noise*noise));
							//double derived = (-(Math.E*rating.rating)/(Math.exp(rating.rating*sij)+Math.E));
							double derived = (Wab.getArray()[b][a]*(1./likeliTradeOff))-getDerivedC(rating.rating, i ,rating.item);
							Item it = items.get(rating.item);
							gradient -= derived*user.getUserFeatures().get(b, 0)*it.getItemFeatures().get(a, 0);
							//Matrix minusPart = (items.get(rating.item).getItemFeatures().tensorProduct(user.getUserFeatures())).times(derived);
						}
					}
					Wab.getArray()[b][a]=Wab.get(b,a)-(gradient*0.00000004*newE);
				}
				
			}
			System.out.println("!");
			System.out.println(Wab.get(0, 0));
			
			temp = 0.;
			double temp2 = 0.;
			
			updateMeanAndVariance();
			for(int i = 0;i<oldW.getRowDimension();i++)
			{
				temp+=Math.abs(oldW.get(i, 0)-Wab.get(i, 0));
			}
			for(int i = 0;i<oldW.getRowDimension();i++)
			{
				temp2+=Wab.get(i, 0);
			}
			System.out.println("Difference is "+temp+" and the average weight is now "+temp2);
			iterations++;
			if(lastTemp<temp) flip*=-1.;
			lastTemp=temp;
		}while(newE<oldE&&iterations<10000);
		
	}
	
	private double getDerivedC(double r, int i, int j) {
		
		return (calculateSij(i, j)-r)/(noise*noise);
		
	}
	private double getDerivedD(double r, int i, int j) {
		return r*(1-calculateProbabilityDiscrete(r, i, j));
		
	}
	private double likelihoodFunctionC(){
		double tempW=0.;
		for (int i = 0; i < Wab.getRowDimension(); i++) {
			for (int j = 0; j < Wab.getColumnDimension(); j++) {
				tempW+=Math.pow(Wab.getArray()[i][j], 2);
			}
			
		}
		tempW*=(1./(2.*likeliTradeOff));
		double returnValue = 0.;
		for(int i = 0;i<users.size();i++){
			for (Rating r : users.get(i).getRatings()) {
				double indicator=calculateSij(i, r.item);
				returnValue +=(- (Math.pow(r.rating-indicator, 2)/(2*noise*noise))-0.5*Math.log(2*Math.PI*noise));
			}
		}
		return tempW-returnValue;
	}
	private double likelihoodFunctionD(){
		double returnValue = 0.;
		for(int i = 0;i<users.size();i++){
			for (Rating r : users.get(i).getRatings()) {
				double indicator=calculateSij(i, r.item);
				returnValue -= Math.log(1+Math.exp(-r.rating*indicator+1));
			}
		}
		return returnValue;
	}


	private void updateMeanAndVariance(){
		for (int i = 0; i < Wab.getRowDimension(); i++) {
			meanWeight+=Wab.getArray()[i][0];
		}
		meanWeight/=Wab.getRowDimension();
		for (int i = 0; i < Wab.getRowDimension(); i++) {
			varianceWeight+=Math.pow((meanWeight-Wab.getArray()[i][0]),2.);
		}
		varianceWeight/=Wab.getRowDimension();
		//varianceWeight*=varianceWeight;
	}
	public double calculateProbabilityContinuous(double rating,int userID,int itemID)
	{
		double sij=calculateSij(userID, itemID);
		//double sij=calculateSij(userID, itemID);
		return (1./Math.sqrt(2*noise*Math.PI))*Math.exp(-(Math.pow(rating-sij,2))/Math.pow(noise, 2));
	}
	public double calculateProbabilityDiscrete(double rating,int userID,int itemID)
	{
		double sij=calculateSij(userID, itemID);
		//Sij=(Wab.transpose()).times(items.get(itemID).getItemFeatures().tensorProduct(users.get(userID).getUserFeatures()));
		return (1./(1+Math.exp(-rating*sij)));
	}


	public void rateItem(int user,int item)
	{
		double predictionPro = calculateProbabilityContinuous(1., user, item);
		double predictionNeg = calculateProbabilityContinuous(0.1, user, item);
		System.out.println("The prediction in favor is :" +predictionPro);
		System.out.println("The prediction against is :" + predictionNeg);
		System.out.println("Check the indicator: "+calculateSij(user, item));
//		for (int i = -10; i < 10; i+=1) {
//			double prediction = calculateProbabilityContinuous((double)i/10., user, item);
//			System.out.println("Value at "+(double)i/10.+" is "+prediction);
//			
//		}
	}
	
	public void rateGetPredictors()
	{
		int counterU = 0;
		for (int j=0;j<users.size();j++) {
			String[] ratingList = new String[items.size()];
			for (int k = 0;k<items.size();k++) {
				ratingList[k] = "Item "+ k +":" + this.calculateSij(j, k) +" | ";
			}
			System.out.println("Ratings user "+j);
			for(int i = 0;i<ratingList.length;i++){
				System.out.print(ratingList[i]);
			}
			System.out.println("_");
		}
	}
	
	private double calculateSij(int i, int j){
		return ((users.get(i).getUserFeatures().transpose().times(Wab)).times(items.get(j).getItemFeatures())).get(0, 0);
		//return (Wab.transpose().times(users.get(i).getUserFeatures().tensorProduct(items.get(j).getItemFeatures()))).get(0, 0);
		//return calculateSijAlt(i, j);
	}
	private double calculateSijAlt(int i, int j) {
		double value = 0.;
		for (int k = 0; k < users.get(i).getUserFeatures().getRowDimension(); k++) {
			for (int l = 0; l < items.get(j).getItemFeatures().getRowDimension(); l++) {
				value+=users.get(i).getUserFeatures().get(k, 0)*items.get(j).getItemFeatures().get(l, 0)*Wab.get(k+(l*users.get(k).getUserFeatures().getRowDimension()), 0);
			}
		}
		return value;
	}
	
	public void rateGetBudgetCorrelation()
	{
		double[][] uiClassMatrix = new double[3][3];
		int[][] uiClassMatrixCounter = new int[3][3];
		String[] simplify = {"low budget","medium budget","high budget"};
		
		
		int budgetClassUser = 0;
		int budgetClassItem = 0;
		for (int i=0;i<users.size();i++) {
			if(users.get(i).getUserFeatures().get(31, 0)==1.) budgetClassUser = 0;
			else if(users.get(i).getUserFeatures().get(32, 0)==1.) budgetClassUser = 1;
			else budgetClassUser = 2;	
			for (Rating rating : users.get(i).ratings) {
				
				Item item = items.get(rating.item);
				
				if(item.getItemFeatures().get(12, 0)==1.) budgetClassItem = 0;
				else if(item.getItemFeatures().get(13, 0)==1.) budgetClassItem = 1;
				else budgetClassItem = 2;
				uiClassMatrixCounter[budgetClassUser][budgetClassItem]+=1;
				uiClassMatrix[budgetClassUser][budgetClassItem]+=calculateProbabilityContinuous(rating.rating, i, rating.item);
				
			}
		}
		for (int i = 0; i < uiClassMatrix.length; i++) {
			for (int j = 0; j < uiClassMatrix[i].length; j++) {
				System.out.println(simplify[i]+" user and "+simplify[j]+ " item scores "+uiClassMatrix[i][j]/(double)uiClassMatrixCounter[i][j]);
			}
			
		}
	}
}
