package model;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import Jama.Matrix;


import org.hibernate.search.annotations.DocumentId;
import org.hibernate.search.annotations.Field;
import org.hibernate.search.annotations.Index;
import org.hibernate.search.annotations.Indexed;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;


@Indexed
public class Citynode implements XmlNode {

	@DocumentId
	public long id; 
	
	private String type; 
	
	private String description; 
	
	private Set<model.Attribute> attributes;
	
	private Set<User> raters = new HashSet<User>(); 

	private Point location;
	
	private Polygon coveringArea; 

	@Field(index=Index.TOKENIZED)
	private String title; 
	
	private String category; 
	
	private String address; 
	
	private String tag; 
	
	private String phoneNumber; 
	
	private String illustation; 
	
	private String openingHours;
	
	private String itemFeatures;
	
	private Matrix itemMatrix;
	
	
	public String getItemFeatures() {
		return itemFeatures;
	}

	public void setItemFeatures(String itemFeatures) {
		this.itemFeatures = itemFeatures;
	}
	
	public Matrix getItemMatrix() {
		return itemMatrix;
	}

	public void setItemMatrix(Matrix itemMatrix) {
		this.itemMatrix = itemMatrix;
	}

	public String getPhoneNumber() {
		return phoneNumber;
	}


	public void setPhoneNumber(String phoneNumber) {
		this.phoneNumber = phoneNumber;
	}


	public String getIllustation() {
		return illustation;
	}


	public void setIllustation(String illustation) {
		this.illustation = illustation;
	}

	public String getOpeningHours() {
		return openingHours;
	}


	public void setOpeningHours(String openingHours) {
		this.openingHours = openingHours;
	}

	public void addUserToRatings(User us){
		this.getRaters().add(us);
	}
	

	public Citynode()
	{ 
		
	}
	

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public Citynode(long id,String desc,Point location)
	{ 
		this.id = id;
		this.description = desc;
		this.location = location;
	}
	
	
	public Set<model.Attribute> getAttributes() {
		return attributes;
	}

	public void setAttributes(Set<model.Attribute> attribute) {
		this.attributes = attribute;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getCategory() {
		return category;
	}

	public void setCategory(String category) {
		this.category = category;
	}


	
	
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Point getLocation() {
		return location;
	}

	public void setLocation(Point location) {
		this.location = location;
	}

	

	public String getTag() {
		return tag;
	}


	public void setTag(String tag) {
		this.tag = tag;
	}

	public Polygon getCoveringArea() {
		return coveringArea;
	}


	public void setCoveringArea(Polygon coveringArea) {
		this.coveringArea = coveringArea;
	}

	public Set<User> getRaters() {
		return raters;
	}


	public void setRaters(Set<User> raters) {
		this.raters = raters;
	}
	

	public Node getXmlRepresentation(Document domdoc) {
		
		Element citynode = domdoc.createElement("citynode"); 
		Element pos = domdoc.createElement("position");
		Element attributes = domdoc.createElement("attributes"); 
		Element raters = domdoc.createElement("raters"); 
		
		
		pos.setAttribute("lat",(new Double((location.getCoordinates()[0].x))).toString()); 
		pos.setAttribute("long",(new Double((location.getCoordinates()[0].y))).toString()); 
		
		citynode.setAttribute("description",description); 
		citynode.setAttribute("title", title); 
		citynode.setAttribute("category",category);
		citynode.setAttribute("type", type); 
		citynode.setAttribute("id",String.valueOf(id));
		citynode.setAttribute("tag", tag);
		citynode.setAttribute("phoneumber",getPhoneNumber());
		citynode.setAttribute("illustation",getIllustation());
		citynode.setAttribute("address", getAddress());
		citynode.setAttribute("openinghours", getOpeningHours()); 
	
		for(Attribute a : this.getAttributes())
		{ 
			attributes.appendChild(a.getXmlRepresentation(domdoc)); 
		}
		
		
		if(this.raters!=null && this.raters.size()>0)
		{
			User u = (User) this.raters.toArray()[0];
			citynode.setAttribute("user", String.valueOf(u.getId()));
			
		}
		
		
		citynode.appendChild(attributes); 
		
		citynode.appendChild(pos);
		
		return citynode; 
	}


	@Override
	public String getNodeType() {
		// TODO Auto-generated method stub
		return "citynode";
	} 
	

	
}

