package recommendersystem;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import model.Citynode;
import model.Context;
import model.Rating;
import model.Recommendation;
import model.User;
import model.UserModel;
import model.Visit;

import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.operation.distance.DistanceOp;

import dao.CitynodeDAO;
import dao.RatingDAO;
import dao.RecommendationDAO;
import dao.UserDAO;
import dao.UserModelDAO;
import dao.VisitDAO;

public abstract class AbstractAIRecommenderSystem implements RecommenderSystem {
	
	protected CitynodeDAO cdao;  
	
	protected RecommendationDAO rdao; 
	
	protected UserDAO udao; 
	
	protected UserModelDAO umdao; 
	
	protected VisitDAO vdao; 
	
	protected RatingDAO ratingdao; 
	
	protected boolean isBinaryFeedback = true; 
	
	protected String currentCategory; 
	
	
	protected abstract List <Recommendation> buildRecommendationList(List <Citynode> nodes,User u,Context c); 
	
	public abstract boolean  handleExplicitFeedback(Citynode n,User user,double feeback);
	
	
	/*
	 * The following should be implemented here : 
	 */
	public abstract List<Recommendation> getRecommendationsForSpesificPoint(User u,
			Point p);  
	
	public abstract User findUser(Context c,long userId,User u);
	
	public abstract UserModel getUserModel(User user);
	
	

	
	public AbstractAIRecommenderSystem(CitynodeDAO cdao,RecommendationDAO rdao,UserDAO udao,UserModelDAO umdao,VisitDAO vdao,RatingDAO ratingdao)
	{ 
		this.cdao = cdao; 
		this.rdao = rdao; 
		this.udao = udao;
		this.umdao = umdao; 
		this.vdao = vdao; 
		this.ratingdao = ratingdao;
		
	}
	
	
	public List<Recommendation> getRecommendations(User user, Context context, String searchstring, boolean filterOnUserPreferences)
	{
	
		/*
		 * 1)Fetch citynodes based on the key words and location
		 * 2)For all citynodes, fetch recommendations (if any) given to the user for the citynode 
		 * 3)Fetch the user model
		 * 4)Generate a compete set of recommendations with a calculated prediction 
		 * 5)Filter on prediction rating  
		 * 5)Set the filtered sub set as the users current recommendations 
		 * 6)Return list of recommendations  
		 */
		
		List<Citynode> nodes = null; 
		User u = udao.findByID(user.getId(), false); 
		
		this.currentCategory = searchstring; 
		
		if(searchstring.trim().equals("Restaurant"))
			nodes = cdao.findCitynodesEat(context);
		else if(searchstring.trim().equals("Hotel"))
			nodes = cdao.findCitynodesSleep(context);
		else if(searchstring.trim().equals("Activity"))
			nodes = cdao.findCitynodeExperience(context); 
		else
		{
			nodes  = new ArrayList<Citynode>(); //cdao.findBySearch(searchstring, null); 
			nodes.add(cdao.findByID((long)40, false));
			
			System.out.println(cdao.findByID((long)40, false));
			
			System.out.println("roter rundt i sk");
		}
		List <Recommendation> returnRecommendations; 
		
		if(filterOnUserPreferences)
		{
			System.out.println("antall noder:" + nodes.size());
			
			returnRecommendations = buildRecommendationList(nodes,u,context); 
			System.out.println("filtrer p recs");
		}
		else 
		{
			System.out.println("roter i noe annet");
			returnRecommendations = new ArrayList<Recommendation>();
			
			for(Citynode n:nodes)
			{ 
			
					Recommendation newRec = new Recommendation();
					
					List <Rating> selected = null; 
					
					newRec.setNode(n); 
					
					double systemRating = 0; 
				
					newRec.setSystemRating((int)(systemRating*100)); 
					
					returnRecommendations.add(newRec);
					newRec.setPersonalized(false);
					
					if(n.getRaters()!=null)
					{
						double sumRating = 0.0;
						int number = 0;
						for(User userRating : n.getRaters())
						{
							if(userRating.getRating(n)!=null)
							{
								number++;
								sumRating+=userRating.getRating(n).getRating();
								
								if(userRating.getRating(n).getRating()>0.7 && selected==null)
								{
									selected = this.getRatingsInArea(userRating.getRatings(),n, context); 
								}
								
							}	
							
						}
						
						newRec.setSystemRating((int)((sumRating/number)*100));
						newRec.setTextualExplanation("Average rating from " + number + " users");
						newRec.setPersonalized(true);
						newRec.setSelectedRatings(selected);
					}
					
				}
		}
		
		u.setCurrentRecommendations(new HashSet<Recommendation>()); 
		
		
		for(Recommendation r : returnRecommendations)
		{
			u.getCurrentRecommendations().add(r);
	
		}
		
		udao.makePersistent(u);
		
		//add user
		for(Recommendation r : returnRecommendations)
		{
			
			r.setUser(u); 
			rdao.makePersistent(r);
		}
		
		//add distance (not persistent information) 
		
		Set <Recommendation> travelPlan = u.getTravelPlan(); 
		
		for(Recommendation r : returnRecommendations)
		{
			double distance = DistanceOp.distance(context.getUserLocation(),r.getNode().getLocation());
			
			r.setDistanceToUser((int)(distance*100.0*1000.0)); 
			
			for(Recommendation travelPlanRec : travelPlan)
			{
				if(travelPlanRec.getNode().getId()==r.getNode().getId())
					r.setIsTravelPlanRecommendation(1); 
			}
			
		}
		
		
	
		
		
		return returnRecommendations; 
	}
	
	
	
	
	protected List <Rating> getRatingsInArea(Set<Rating> ratings, Citynode n, Context c)
	{
		
		List <Rating> returnList = new ArrayList<Rating>(); 
		
		for(Rating r : ratings)
		{
			double distance = DistanceOp.distance(c.getUserLocation(),r.getCitynode().getLocation())*(100.0*1000.0);
				
			
			if(r.getCitynode().getId()!=n.getId() && distance<10000)
				returnList.add(r); 
		}
		
		if(returnList.size()>0)
			return returnList; 
		else 
			return null; 
	}
	
	
	
	
	


	@Override
	public Recommendation addRecommendationToPlan(User user,
			long recommendationId) {
		
		user = udao.findByID(user.getId(), false); 
		
		Recommendation recommendation = rdao.findByID(recommendationId, false);
		
		
		
		recommendation.setIsTravelPlanRecommendation(1); 
		user.getTravelPlan().add(recommendation); 
		udao.makePersistent(user); 
		
		

		return recommendation;
	}


	  

	public List<Recommendation> getTravelPlan(User u, Context context)
	{ 
		
		//no activities later than 
		int bedTime = 21; 
		u = udao.findByID(u.getId(), false); 
		
		
		List <Recommendation> travelPlan = new ArrayList<Recommendation>();
		
		List <Recommendation> resturants = getRecommendations(u, context, "Restaurant", true); 
		List <Recommendation> activities= getRecommendations(u, context, "Activity", true); 
		
		/*
		Recommendation rec1 = new Recommendation(); 
		rec1.setNode(cdao.findByID((long)3, false)); 
		rec1.setSystemRating(80); 
		rec1.setDistanceToUser(912); 
		rec1.setTextualExplanation("Average rating from 15 people that rated Notredam cathedral and effil tower similary as you");
		
		Recommendation rec2 = new Recommendation();
		
		rec2.setNode(cdao.findByID((long)29, false)); 
		rec2.setSystemRating(90); 
		rec2.setDistanceToUser(2411); 
		rec2.setTextualExplanation("Average rating from 20 people that rated the effil tower and family resturants similary as you");
		
		Recommendation rec3 = new Recommendation();
		rec3.setNode(cdao.findByID((long)31, false)); 
		rec3.setSystemRating(90); 
		rec1.setDistanceToUser(1200); 
		rec3.setTextualExplanation("Average rating from 20 people that rated history similary as you");
		
		Recommendation rec4 = new Recommendation();
		rec4.setNode(cdao.findByID((long)28, false)); 
		rec4.setSystemRating(90); 
		rec1.setDistanceToUser(1100); 
		rec4.setTextualExplanation("Average rating from 7 people that rated typical english pubs similary as you");
		
		travelPlan.add(rec1); 
		travelPlan.add(rec2); 
		travelPlan.add(rec3);
		travelPlan.add(rec4);
		
		*/
		
		
		boolean hasDinnerRecommendation = false; 
		boolean hasLunchRecommendation = false; 
		
		Date tempTime = new Date() ;//TODO: use usertime context.getUsertime();
		Point tempLocation = context.getUserLocation(); 
		
		tempTime.setHours(8);
		int itterations = 0; 
		
		while(tempTime.getHours()<bedTime && itterations<20)
		{ 
			itterations++; 
			System.out.println("Evig lkke" + itterations);
			//if dinner time , and not yet had dinner ->try to find dinner 
			if(tempTime.getHours()>16 && tempTime.getHours()<21 && !hasDinnerRecommendation)
			{ 
				Recommendation dinner = pickRecommendation(tempLocation, resturants); 
				if(dinner!=null && !conationsRecommendation(travelPlan,dinner))
				{ 
					resturants.remove(dinner);
					travelPlan.add(dinner); 
					tempLocation = dinner.getNode().getLocation(); 
					//TODO : Find a better estimate for this 
					tempTime.setHours(tempTime.getHours()+2); 
					
					hasDinnerRecommendation = true; 
					
				}
			}
			if(tempTime.getHours()>11 && tempTime.getHours()<14 && !hasLunchRecommendation)
			{ 
				Recommendation lunch = pickRecommendation(tempLocation, resturants); 
				if(lunch!=null && !conationsRecommendation(travelPlan,lunch))
				{ 
					resturants.remove(lunch);
					travelPlan.add(lunch); 
					tempLocation = lunch.getNode().getLocation(); 
					//TODO : Find a better estimate for this 
					tempTime.setHours(tempTime.getHours()+2); 
					
					hasLunchRecommendation = true; 
					
				}
			}
			if(tempTime.getHours()<bedTime)
			{

				Recommendation activity =  pickRecommendation(tempLocation, activities); 
				if(activity!=null && !conationsRecommendation(travelPlan, activity))
				{
					travelPlan.add(activity); 
					activities.remove(activity);
					tempLocation = activity.getNode().getLocation(); 
					//TODO : Find a better estimate for this 
					tempTime.setHours(tempTime.getHours()+2); 
					
				}
			}
		
		}
		
		
		
		Set<Recommendation> travelPlanToDb = new HashSet<Recommendation>();
		
		for(Recommendation recommendation : travelPlan)
		{ 
			recommendation.setIsTravelPlanRecommendation(1); 
			travelPlanToDb.add(recommendation); 
		}
		
		u.setRecivedTravelPlan(new Date()); 
		u.setTravelPlan(travelPlanToDb); 
		
		udao.makePersistent(u); 
		
		for(Recommendation r : travelPlan)
		{
			double distance = DistanceOp.distance(context.getUserLocation(),r.getNode().getLocation());
			
			r.setDistanceToUser((int)(distance*100.0*1000.0)); 
			
		
			
		}
		
		
		
		return travelPlan; 
	}

	private boolean conationsRecommendation(List <Recommendation> recs, Recommendation rec)
	{
		for(Recommendation r : recs)
		{
			if(r.getId()==rec.getId())
				return true;
		}
		
		return false; 
	}
	
	//finds the closet recommendation above the threshold
	private Recommendation pickRecommendation(Point tempPoint, List <Recommendation> recs)
	{ 
		Recommendation bestSoFar = null; 
		int threshHold = 60; 
		
		for(Recommendation r : recs)
		{ 
			if(bestSoFar==null)
			{
				bestSoFar = r; 
			}
			else 
			{
				double newDistance = DistanceOp.distance(tempPoint,r.getNode().getLocation());
				double oldDistance = DistanceOp.distance(bestSoFar.getNode().getLocation(),tempPoint);
				
				if(newDistance<oldDistance && bestSoFar.getSystemRating()>threshHold)
				{ 
					bestSoFar = r; 
				}
			}
		}
		return bestSoFar; 
	}
	


	@Override
	public Recommendation removeRecommendationFromList(User user,
			long recommendationId) {
	
		user = udao.findByID(user.getId(), false); 
		
		Recommendation recommendation = rdao.findByID(recommendationId, false);
		

		
		
			
		boolean remove = false; 
		Recommendation recToRemove = null; 
		
		for(Recommendation r : user.getTravelPlan())
		{
				if(r.getNode().getId()==recommendation.getNode().getId())
				{ 
					remove = true; 
					recToRemove = r; //user.getTravelPlan().remove(r); 
				}
			
		}
		if(remove)
		{
			user.getTravelPlan().remove(recToRemove); 
		}
			
		user.setTravelPlan(user.getTravelPlan()); 
			
		recommendation.setIsTravelPlanRecommendation(0); 
		udao.makePersistent(user); 
			
		
		recommendation.setIsTravelPlanRecommendation(0); 
			
		return recommendation; 
	
		

		
	}

	@Override
	public boolean removeTravelPlan(User user) {
		
		user = udao.findByID(user.getId(), false);
		
		user.setTravelPlan(new HashSet<Recommendation>()); 
		udao.makePersistent(user); 
		
		return true;
	}
	
	

	
	
	
	/* TDOD : Do something cooler 
	 * TODO : Handle more than one "visting nodes"(right now we just pick the first one) 
	if this method manages to derive feedback, it will automatically propagate this to change the users preference model
	*/
	//
	public Citynode handleImplicitFeedback(User user,Context context) 
	{
		
		
		
		boolean hasDerivedFeedback = false; 
		Citynode nodeApplicableForFeeback = null; 
		
		user = udao.findByID(user.getId(), false); 
		 
		 //TODO : Handle more than one node
		 List <Citynode> l  = cdao.contains(context.getUserLocation()); 
		 
		 cdao.findByID((long)11,false);
		Citynode currentNode = l==null?null:l.get(0); 
		 
		 System.out.println("current node :" + currentNode);
		 
		 
		 Visit currentVisit = user.getCurrentVisit(); 
		 
	
		 
		 Date sysTime = new Date(); 
		
			 //Find the approximately spent time and set the visit to null (also generate a feedback )  
			 if(currentNode==null)
			 { 
				 if(currentVisit!=null)
				 {
					 Date arrived = currentVisit.getArrived();
					 
					 //work around (can't seem to store full date, only time stamp)
					 arrived.setDate(sysTime.getDate()); 
					 arrived.setYear(sysTime.getYear());
					 arrived.setMonth(sysTime.getMonth());
					 
					 
					 long vistingTime = currentVisit.getTimeSpent() + (sysTime.getTime()-arrived.getTime()); 
					
					 currentVisit.setTimeSpent(vistingTime); 
					 vdao.makePersistent(currentVisit);
					 //If the user spent more than 10 minutes, we can assume he or she liked it(?)  
					 if(((vistingTime/1000)/60)>5)
					 {
					//	 propagateFeedback(currentVisit.getPlace().getAttributes(), user, 1.0); 
						 hasDerivedFeedback = true; 
						 nodeApplicableForFeeback = currentVisit.getPlace(); 
					 }
					 user.setCurrentVisit(null); 
					 
					 //return currentNode;
					
				 }
				
			}
			 else 
			 {
			
				if(currentVisit!=null)
				{
					
					Date arrived = currentVisit.getArrived();
					 
					 //work around (can't seem to store full date, only time stamp)
					 arrived.setDate(sysTime.getDate()); 
					 arrived.setYear(sysTime.getYear());
					 arrived.setMonth(sysTime.getMonth());
					
					 if(currentNode.getId()==currentVisit.getPlace().getId())
					 {
						 currentVisit.setTimeSpent(sysTime.getTime()-arrived.getTime()); 
						 this.vdao.makePersistent(currentVisit); 
						 long vistingTime = currentVisit.getTimeSpent(); 
						 if(((vistingTime/1000)/60)>5)
						 {
							 hasDerivedFeedback = true; 
							 nodeApplicableForFeeback = currentVisit.getPlace(); 
							// propagateFeedback(currentVisit.getPlace().getAttributes(), user, 1.0); 
						 }
					 }
					 else 
					 {
						 long vistingTime = currentVisit.getTimeSpent(); 
						 if(((vistingTime/1000)/60)>5)
						 {
							 hasDerivedFeedback = true; 
							 nodeApplicableForFeeback = currentVisit.getPlace(); 
							// propagateFeedback(currentVisit.getPlace().getAttributes(), user, 1.0); 
						 }
						 
						 Visit v = new Visit(); 
						 Date d = new Date();    
						 v.setArrived(new Date());
						 v.setTimeSpent(0);
						 v.setPlace(currentNode); 
						 this.vdao.makePersistent(v); 
						 user.setCurrentVisit(v); 
						
					 }
				}
				else 
				{ 
					 Visit v = new Visit(); 
					 Date d = new Date();    
					 v.setArrived(new java.sql.Date(d.getTime()));
					 v.setTimeSpent(0);
					 v.setPlace(currentNode); 
					 
					 user.setCurrentVisit(v); 
					
					 this.vdao.makePersistent(v); 
				}
			 }
			 
		 udao.makePersistent(user); 
			 
		 System.out.println("this is node applicable for feedback:" + nodeApplicableForFeeback);
		 
	     return nodeApplicableForFeeback; 
	}
		

	public List<Recommendation> handleContextChange(User user, Context context) {
			
			List <Recommendation> returnList = new ArrayList<Recommendation>(); 
			//if implicit feedback is found ..
			
			user = this.udao.findByID(user.getId(), false); 
			
			if(!context.isCheckingForTravelPlan())
				returnList = checkForRecommendations(user, context); 
			else 
				returnList = checkForTravelPlan(user, context); 
			
			return returnList; 
			 
		}
		
	
	public Recommendation getRecommendationFromCityNode(long nodeId, User c,Context context)
	{
		Citynode n = cdao.findByID(nodeId,false); 
	
		
		Recommendation newRec = new Recommendation();
		
		List <Rating> selected = null; 
		
		newRec.setNode(n); 
		
		double systemRating = 0; 
	
		newRec.setSystemRating((int)(systemRating*100)); 
		
		
		newRec.setPersonalized(false);
		
		if(n.getRaters()!=null)
		{
			double sumRating = 0.0;
			int number = 0;
			for(User userRating : n.getRaters())
			{
				if(userRating.getRating(n)!=null)
				{
					number++;
					sumRating+=userRating.getRating(n).getRating();
					
					if(userRating.getRating(n).getRating()>0.7 && selected==null)
					{
						selected = this.getRatingsInArea(userRating.getRatings(),n, context); 
					}
					
				}	
				
			}
		
			newRec.setSystemRating((int)((sumRating/number)*100));
			newRec.setTextualExplanation("Average rating from " + number + " users");
			newRec.setPersonalized(true);
			newRec.setSelectedRatings(selected);
		}
		
		
		return newRec;
		
	}
	
	
	/*
 	* TODO : Filter the resulting recommendations on rating (right now we are returning everything) 
 	* 
 	* This method serve two  purposes :  1) Find out if it is possible to derive implicit feedback from the users movement 
	 * 									 2) Find out if there are interesting places nearby we can send to the user. 
	 * 							 		
	 */  
private List<Recommendation> checkForRecommendations(User user, Context context)
{ 
	
	List <Recommendation> recs = new ArrayList<Recommendation>(); 

	List <Recommendation> returnList = new ArrayList<Recommendation>(); 
	
		//..we should look for new recommendations close to the user  (using radius of 40 meters for testing purposes )
	recs = buildRecommendationList(cdao.findCityNodeWithin(context.getUserLocation(), 60.0),user,context);  
		//TODO : filter on  rating .. 
	
	for(Recommendation r : recs)
	{
		boolean removeRec = false; 
		
		for(Recommendation r2 : user.getCurrentRecommendations())
		{
			if(r.getNode().getId()==r2.getNode().getId())
				removeRec = true; 
		}
		
		returnList.add(r); 
	}
	
	if(user.getCurrentRecommendations()==null)
		user.setCurrentRecommendations(new HashSet<Recommendation>()); 
	
	
	for(Recommendation r : returnList)
	{
		user.getCurrentRecommendations().add(r);

	}
	
	udao.makePersistent(user);
	// .. and add distance (not persistent information) 
	//TODO : This code can be place somewher else , move it .. 
	for(Recommendation r : returnList)
	{
		double distance = DistanceOp.distance(context.getUserLocation(),r.getNode().getLocation());
		r.setDistanceToUser((int)(distance*100.0*1000.0)); 
		r.setIsProactive(1); 
	}
	
	return returnList; 
}

/*
 * If the user has not recvied at travel plan today , generate a new one 
 * and relate it to the user in db
 */
	private List<Recommendation> checkForTravelPlan(User user, Context context)
	{ 
		
		user = udao.findByID(user.getId(), false);
		List <Recommendation> returnRecs = new ArrayList<Recommendation>(); 
		/*
		Date userTime = new Date(); //TODO: use time from context
		//if(user.getRecivedTravelPlan().getDay()!=userTime.getDay())
		//{ 
			returnRecs = this.getTravelPlan(user, context); 
		//}
		else 
		{
			returnRecs.addAll(user.getTravelPlan()); 
		}
		*/
		returnRecs.addAll(user.getTravelPlan()); 
		
		for(Recommendation r : returnRecs)
		{
			double distance = DistanceOp.distance(context.getUserLocation(),r.getNode().getLocation());
			r.setDistanceToUser((int)(distance*100.0*1000.0)); 
			
			//r.setIsProactive(1); 
		}
		
		
		return returnRecs;
	
	
	
	}


	public User createUser()
	{ 
		User u = new User();
		
		udao.makePersistent(u); 
		
		return u;
	}

	
	public List <Citynode> getInitSet()
	{
		
		List <Citynode> startList = new ArrayList<Citynode>(); 
		
		for(int i=12;i<22;i++)
		{
			startList.add(cdao.findByID((long)i, false)); 
		}
		//startList.add(cdao.findByID((long)32, false)); 
		///List <Citynode> nodes = cdao.findAll(); 
		
		return startList;
		
	}
	
	

}
