package recommendersystem.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import matrixPacket.Item;
import matrixPacket.RecommenderAlgorithm;
import model.Citynode;
import model.Rating;
import model.Recommendation;
import model.User;
import Jama.Matrix;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;

public class TestingRecommender {
	static int nameCounter = 0;
	static int citynodeTitleCounter = 0;
	static int locationCounter = 0;
	
	
	public static User createPseudoUser(){
		User us = new User();
		us.setUsername("user" + String.valueOf(nameCounter));
		int randR = (int) Math.round(Math.random()*3);
		if (randR < 1.)
			us.setUserFeatures("0100011000000000100100000000000010");
		else if (randR < 2.)
			us.setUserFeatures("0100011000000000100100000000000100");
		else
			us.setUserFeatures("0100011000000000100100000000000001");
			
		nameCounter++;
		
		return us;
	}
	
	public static Point createPseudoPoint(){
		String wktPoint = "POINT(23." + String.valueOf(locationCounter) + "313 21.23123)";
		//First interpret the WKT string to a point
        WKTReader fromText = new WKTReader();
        Geometry geom = null;
        try {
            geom = fromText.read(wktPoint);
        } catch (ParseException e) {
            throw new RuntimeException("Not a WKT string:" + wktPoint);
        }
        if (!geom.getGeometryType().equals("Point")) {
            throw new RuntimeException("Geometry must be a point. Got a " + geom.getGeometryType());
        }
		
        locationCounter++;
		
		return (Point) geom; 
		
	}
	
	public static Citynode createPseudoCitynodeRestaurant(){
		Citynode cn = new Citynode();
		cn.setTitle("citynode" + String.valueOf(citynodeTitleCounter));
		cn.setLocation(createPseudoPoint());
		cn.setItemFeatures("100000000000100");
		
		citynodeTitleCounter++;
		
		return cn;
	}
	
	public static Citynode createPseudoCitynodeHotel(){
		Citynode cn = new Citynode();
		cn.setTitle("citynode" + String.valueOf(citynodeTitleCounter));
		cn.setLocation(createPseudoPoint());
		cn.setItemFeatures("001100000000100");
		
		citynodeTitleCounter++;
		
		return cn;
	}
	
	public static Citynode createPseudoCitynodeExperience(){
		Citynode cn = new Citynode();
		cn.setTitle("citynode" + String.valueOf(citynodeTitleCounter));
		cn.setLocation(createPseudoPoint());
		cn.setItemFeatures("001000001000100");
		
		citynodeTitleCounter++;
		
		return cn;
	}
	
	public static Rating createPseudoRating(User us, Citynode cn){
		Rating r = new Rating();
		
		int randR = (int) Math.round(Math.random()*3);
		
		if(us.getUserFeatures().charAt(31) == '1'){
			if(cn.getItemFeatures().charAt(12) == '1')
				r.setRating(1.0);
			else if(cn.getItemFeatures().charAt(13) == '1') 
				r.setRating(0.5);
			else 
				r.setRating(.1);
			
		} else if (us.getUserFeatures().charAt(32) == '1'){
				if(cn.getItemFeatures().charAt(12) == '1')
					r.setRating(0.6);
				else if(cn.getItemFeatures().charAt(13) == '1') 
					r.setRating(1.0);
				else 
					r.setRating(.3);
			
		} else {
				if(cn.getItemFeatures().charAt(12) == '1')
					r.setRating(0.1);
				else if(cn.getItemFeatures().charAt(13) == '1') 
					r.setRating(0.4);
				else 
					r.setRating(1.0);
			
		}
		
		r.setCitynode(cn);
		
		us.addRating(r);
		cn.addUserToRatings(us);
		
		return r;
	}

	public static void main(String[] arg){
		
		ArrayList<User> users = new ArrayList<User>();
		for (int i = 0; i < 200; i++) {//200
			User u = createPseudoUser();
//			System.out.println(u.getUsername());
			users.add(u);
		}
		

		ArrayList<Citynode> nodes = new ArrayList<Citynode>();
		for (int i = 0; i < 50; i++) {//50
			Citynode cnR = createPseudoCitynodeRestaurant();
			nodes.add(cnR);
			Citynode cnH = createPseudoCitynodeHotel();
			nodes.add(cnH);
			Citynode cnE = createPseudoCitynodeExperience();
			nodes.add(cnE);
		}
		
		
		for(int i = 0; i< 1000; i++) {//1000
			int randU = (int) Math.round(Math.random()*199);//199
			try {
				Thread.sleep(1);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			int randI = (int) Math.round(Math.random()*49);//49
			Rating r = createPseudoRating(users.get(randU), nodes.get(randI));
//			users.get(randU).addRating(r);
		}
		
		
		
		// Creation and conversion of necessary entities
		ArrayList<matrixPacket.User> userList = new ArrayList<matrixPacket.User>();
		Set<matrixPacket.User> tempUserList = new HashSet<matrixPacket.User>();
		ArrayList<Item> itemList = new ArrayList<Item>();
		
		// Get Citynodes/Items
		System.out.println("Getting items...");
		for(Citynode node : nodes){
			// Get features from node and create an AI-Item based on those
//			items.add(new matrixPacket.Item(node.getItemMatrix()));
			
			itemPortToMatrix(node);
			
			matrixPacket.Item it = new matrixPacket.Item(node.getItemMatrix());
			it.setCitynode(node);
			
			itemList.add(it);
		}
		
		
		// Get users
		System.out.println("Getting users...");
		for(Citynode node : nodes)
		{
			
			Set <User> raters = node.getRaters(); 
			for(User rater : raters)
			{
				// Get features from user and create an AI-User based on those
				userPortToMatrix(rater);
				
				matrixPacket.User mUser = new matrixPacket.User();
				mUser.setUserFeatures(rater.getUserMatrix());
				mUser.modelUser = rater;
				
				// Convert ratings
				for(Rating r : rater.ratings){
					int item = 0;
					
					// Search for index of Item/Citynode
					for(Item it : itemList){
						if(it.getCitynode().getLocation().equals(r.getCitynode().getLocation())){
							item = itemList.indexOf(it);
							it.setItem(item);
							System.out.println("item: " + item);
							break;
						}
					}
					
					double rating = r.getRating();
					
					System.out.println("Adding rating: " + rating);
					mUser.addRating(new matrixPacket.Rating(item, rating));
				}
				
				// Add user if not already in the list
				boolean isPresent = false;
				for(matrixPacket.User user : userList){
					if (user.modelUser!=null){
						if (user.modelUser.isEqual(mUser.modelUser)){
							isPresent = true;
							break;
						}
					}
				}
				
				if (!isPresent){
					userList.add(mUser);
				}
				
				
				
				
			}
		}
		
		System.out.println("Size of userList: " + userList.size());
		System.out.println("Size of itemList: " + itemList.size());
		
		

		RecommenderAlgorithm rec = new RecommenderAlgorithm(userList, itemList);

		rec.optimizeWeights();



		// Traverse users
		for (int i = 0; i < userList.size(); i++) {
			// Traverse items
			for (int j = 0; j < itemList.size(); j++) {
				System.out.println("Prediction user "+i+" on item "+j+ ": ");
				rec.rateItem(i, j);
			}
		}

		rec.rateGetBudgetCorrelation();
		matrixPacket.User temp = userList.get(0);
		for (matrixPacket.Rating ratings : temp.ratings) {
			System.out.println("True rating is "+ratings.rating);
			for (int i = -10; i < 10; i++) {
				System.out.println("Predicted rating is "+rec.calculateProbabilityContinuous((double)i/10., 0, ratings.item)+" for " +(double)i/10.);
			}
			System.out.println(rec.calculateProbabilityDiscrete(1, 0, ratings.item)+"Actual rating was "+ratings.rating);
		}
		
		RatingComparator rc = new RatingComparator();
		Collections.sort(temp.ratings, rc);
		
		// Find highest rating
		double highestScore = 0;
		int itemWHS = 0;
		for(matrixPacket.Rating r : temp.ratings) {
			if(r.rating>highestScore){
				highestScore = r.rating;
				itemWHS = r.item;
			}
		}
		
		Citynode recCN = new Citynode();
		// Find corresponding Citynode
		for(Item it : itemList){
			if(it.getItem()==itemWHS){
				recCN = it.getCitynode();
			}
		}
		
		List <Recommendation> retur = new ArrayList<Recommendation>(); 
		
		Recommendation rr = new Recommendation();
		rr.setNode(recCN);
		retur.add(rr);
		
		System.out.println("Recommended CN Title:" + recCN.getTitle());
		
		
		// Code for several items when applicable
//		System.out.println("List size" + temp.ratings.size());
//		
//		int numberOfRecommendationsToBeSent = 6;
//		for(int i=0; i<numberOfRecommendationsToBeSent; i++){
//			try{
//				Citynode recCN = new Citynode();
//				// Find corresponding Citynode
//				for(Item it : itemList){
//					if(it.getItem()==temp.ratings.get(i).item){
//						recCN = it.getCitynode();
//					}
//				}
//
//
//				Recommendation rr = new Recommendation();
//				rr.setNode(recCN);
//
//				retur.add(rr);
//
//
//				System.out.println("Recommended CN Title:" + recCN.getTitle());
//			} catch (IndexOutOfBoundsException e){
//				
//			}
//		}
		
		
		
	}
	
	
	
	
	// Ports user, from userFeatures (String) to userMatrix (Matrix)
	public static void userPortToMatrix(User us){
		Matrix m = new Matrix(new double[us.getUserFeatures().length()][1]);
		
		for(int x=0; x < us.getUserFeatures().length(); x++){
			if (us.getUserFeatures().substring(x, x+1).equals("1")){
				m.set(x, 0, 1);
			} else {
				m.set(x, 0, 0);
			}
		}
		
		us.setUserMatrix(m);
	}
	
	
	// Ports item, from itemFeatures (String) to itemMatrix (Matrix)
	public static void itemPortToMatrix(Citynode cn){
		Matrix m = new Matrix(new double[cn.getItemFeatures().length()][1]);
		
		for(int x=0; x < cn.getItemFeatures().length(); x++){
			if (cn.getItemFeatures().substring(x, x+1).equals("1")){
				m.set(x, 0, 1);
			} else {
				m.set(x, 0, 0);
			}
		}
		
//		// Print item matrix
//		for(int i=0; i<m.getRowDimension(); i++){
//			System.out.println("Itemsmatrix at " + i + ": " + m.get(i, 0));
//		}
		
		cn.setItemMatrix(m);
	}
	
	
	
	
	
	
}
