package web;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.WKTReader;
import com.vividsolutions.jts.io.ParseException;

import util.HibernateUtil;
import org.hibernate.Session;

import dao.CitynodeDAO;
import model.Citynode;;


public class InsertPOIServlet extends HttpServlet {
	
	public CitynodeDAO cdao;
	
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		
		// Obtain parameters from web-form
		String []parameters = new String[7];
		parameters[0] = request.getParameter("title");
		parameters[1] = request.getParameter("address");
		parameters[2] = request.getParameter("description");
		parameters[3] = request.getParameter("phonenumber");
		parameters[4] = request.getParameter("openinghours");
		parameters[5] = request.getParameter("category");
		parameters[6] = request.getParameter("type");
		
		// Check that none is blank (results in '' in database)
		for(int i=0; i<parameters.length; i++){
			if(parameters[i].isEmpty())
				parameters[i] = null;
		}
		
		// Get location
		String latStr = request.getParameter("googleLat");
		String lngStr = request.getParameter("googleLng");
		String location = "POINT(" + request.getParameter("location") + ")";
		
		// Derive area from location
		double lat = Double.parseDouble(latStr);
		double lng = Double.parseDouble(lngStr);
		
		// Initialize variables
		double latDifference = 0.000025;
		double lonDifference = 0.00005;
		
		// Get position for each corner of the box
		double boxLat1 = lat + latDifference;
		double boxLat2 = lat + latDifference;
		double boxLat3 = lat - latDifference;
		double boxLat4 = lat - latDifference;
		
		double boxLon1 = lng - lonDifference;
		double boxLon2 = lng + lonDifference;
		double boxLon3 = lng + lonDifference;
		double boxLon4 = lng - lonDifference;
		
		String polygon = "POLYGON((" + boxLat1 + " " + boxLon1 + ", " + boxLat2 + " " + boxLon2 + ", "
		 + boxLat3 + " " + boxLon3 + ", " + boxLat4 + " " + boxLon4 + ", " + boxLat1 + " " + boxLon1 + "))";

        //First interpret the WKT string to a point
        WKTReader fromText = new WKTReader();
        Geometry point = null;
        Geometry area = null;
        try {
            point = fromText.read(location);
            area = fromText.read(polygon);
        } catch (ParseException e) {
            throw new RuntimeException("Not a WKT string:" + location);
        }
        if (!point.getGeometryType().equals("Point")) {
            throw new RuntimeException("Geometry must be a point. Got a " + point.getGeometryType());
        }

        Session session = HibernateUtil.getSessionFactory().getCurrentSession();

        session.beginTransaction();

        // Assign values to citynode
        Citynode cn= new Citynode();
        cn.setTitle(parameters[0]);
        cn.setAddress(parameters[1]);
        cn.setDescription(parameters[2]);
        cn.setPhoneNumber(parameters[3]);
        cn.setOpeningHours(parameters[4]);
        cn.setCategory(parameters[5]);
        cn.setType(parameters[6]);
        cn.setLocation((Point) point);
        cn.getLocation().setSRID(4326);
        cn.setCoveringArea((Polygon)area);
        cn.getCoveringArea().setSRID(4326);
        
        session.save(cn);

        session.getTransaction().commit();
        
        // Print data (optional)
		System.out.println("Citynode added: " + parameters[0]);
		
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		out.println(parameters[0] + " were added. To add more, click <a href=\"insert.html\">here</a>.");
		
		out.close();
    }
}
